# main.py -- put your code here!

from umqttsimple import MQTTClient, MQTTException
import time
import ubinascii
from machine import WDT
import network
import os
import config

last_message = 0
message_interval = 5
counter = 0
ringFlag = False
wdt = None
wdtCnt = 0
client = None
wlan = None


def init():
    global wdt
    wdt = WDT(timeout=8000)
    wdt.feed()
    print("Watchdog enabled")

    global wlan
    wlan = network.WLAN(network.STA_IF)
    wlan.active(True)
    wlan.connect(config.SSID, config.WPA_KEY)

    connectCnt = 0
    while (not wlan.isconnected() or connectCnt > config.MAX_CONNECT_RETRY_CNT):
        print("Not yet connected")
        wdt.feed()
        connectCnt += 1
        time.sleep(1)

    if (not wlan.isconnected()):
        print("Failed to connect to wlan, restarting")
        machine.reset()

    print(f"Connected to WLAN: {machine.reset_cause()} {connectCnt} {wlan.ifconfig()}")

    global client
    clientId = ubinascii.hexlify(bytearray(os.urandom(16))).decode()
    client = MQTTClient(clientId, config.MQTT_SERVER, keepalive=60, 
        ssl=True, user=config.MQTT_LOGIN, password=config.MQTT_PASSWORD)
    client.set_callback(mqtt_callback)
    client.connect()
    print(f"Connected to MQTT broker {config.MQTT_SERVER} ")
    client.subscribe(config.WATCHDOG_TOPIC)
    print(f"Subscribed to {config.WATCHDOG_TOPIC}")
    client.publish(config.STARTUP_TOPIC, f"Hello, rpipw thermometer is alive now at {wlan.ifconfig()}")


def mqtt_callback(topic, msg):
    global wdt, wdtCnt
    # print(f"Received: {topic}: {msg}")
    if topic == config.WATCHDOG_TOPIC:
        wdtCnt += 1
        # print(f"feed watchdog {wdtCnt}")
        wdt.feed()

def main():
    global client
    print("Hello")
    time.sleep(1)

    try:
        init()
    except Exception as e:
        print(f"Some error when starting: {e.__class__.__name__}, {e}")
        time.sleep(15)
        machine.reset()

    last_time_mqtt_loop = time.ticks_ms()
    last_time_1_second = time.ticks_ms()
    last_time_main_period = time.ticks_ms()
    payload_cnt = 0
    while True:
        current_time = time.ticks_ms()

        if (time.ticks_diff(current_time, last_time_mqtt_loop) > 250):
            last_time_mqtt_loop = current_time
            client.ping()
            client.wait_msg()

        if (time.ticks_diff(current_time, last_time_1_second) > 1000):
            last_time_1_second = current_time
            client.publish(config.HEARTBEAT_TOPIC, f"still alive {current_time}")

        if (time.ticks_diff(current_time, last_time_main_period) > config.MAIN_PERIOD_MS):
            last_time_main_period = current_time
            client.publish(config.PAYLOAD_TOPIC, f"measurement {payload_cnt}")
            payload_cnt += 1
            print("Payload sent")


    client.disconnect()
    
if __name__ == "__main__":
    main()